type Getter<T> = () => T;
type Setter<T> = (value: T) => void;
type Accessor<T> = {
    get: Getter<T>;
    set: Setter<T>;
};
interface Signal<T> {
    (): T;
    (value: T): void;
    (value: Partial<T>, patch: boolean): void;
}
type Reactive<T> = Signal<T> & (T extends object ? {
    readonly [key in keyof T]: Reactive<T[key]>;
} & Readonly<T> : {});
type ReadonlyReactive<T> = Getter<T> & (T extends object ? {
    readonly [key in keyof T]: ReadonlyReactive<T[key]>;
} & Readonly<T> : {});
type ReactiveMap<T> = Map<keyof T, Reactive<T[keyof T]>>;
type EffectFunction<T extends any[] = [], R = void> = (...args: T) => R;
type Option<T> = {
    reactiveMap: ReactiveMap<T>;
    parent: Option<T> | null;
    effects: Set<EffectFunction>;
    path: string;
    get: Getter<T>;
    set: Setter<T> | ((value: Partial<T>, patch: boolean) => void);
};
type ReactiveType<T> = T extends Reactive<infer V> ? V : never;
type DependenciesType<T> = T extends [infer F, ...infer N] ? [ReactiveType<F>, ...DependenciesType<N>] : [];
type Getters<T = Record<string | symbol, any>> = {
    [key in keyof T]: Getter<T[key]>;
};
type Actions = {
    [key: string]: (...args: any[]) => void;
};
type Options<Id extends string, S extends Record<string | symbol, any>, G extends Getters, A extends Actions> = {
    id: Id;
    state: S;
    getters: G & ThisType<Store<Id, S, G, {}>>;
    actions: A & ThisType<Store<Id, S, G, A>>;
};
type Computes<T extends Getters<Record<string | symbol, any>>> = {
    readonly [key in keyof T]: () => ReturnType<T[key]>;
};
type Store<Id extends string, S extends Record<string | symbol, any>, G extends Getters, A extends Actions> = {
    $id: Id;
} & Reactive<S> & Computes<G> & Readonly<A>;

declare function read<T>({ effects, get }: Option<T>): T;
declare function write<T>({ effects, reactiveMap, parent, set }: Option<T>, value: T, patch: boolean): void;
declare function triggerParent(option: Option<any> | null): void;
declare function triggerChildren<T>(reactiveMap: ReactiveMap<T>): void;
declare function createSignal<T>(option: Option<T>): Signal<T>;

declare function createProxy<T>(signal: Signal<T>, option: Option<T>): Reactive<T>;
declare function createReactive<T>(path: string, parent: Option<any> | null): Reactive<T>;
declare function useReactive<T>(value: T): Reactive<T>;

declare function clear(): void;
declare function track(effects: Set<EffectFunction>): boolean;
declare function trigger(effects: Set<EffectFunction>): void;
declare function useEffect<T extends [], R>(effect: EffectFunction<T, R>, ...args: T): R;

declare function useComputed<T>(getter: Getter<T>): ReadonlyReactive<T>;
declare function useComputed<T>(accessor: Accessor<T>): Reactive<T>;

declare function useMemo<T>(getter: Getter<T>): Getter<T>;

declare function watch<T extends Reactive<any>[]>(cb: (...values: DependenciesType<T>) => void, ...deps: T): void;

declare const state: any;
declare const getId: () => string;
declare function parsePath(path: string): {
    get(): any;
    set<T>(value: T, patch?: boolean): void;
};

declare function defineStore<Id extends string, S extends Record<string | symbol, any>, G extends Getters<{}>, A extends Actions>(options: Options<Id, S, G, A>): Store<Id, S, G, A>;
declare function defineStore<Id extends string, S extends Record<string | symbol, any>, G extends Getters<{}>, A extends Actions, Context extends Store<Id, S, G, A>>(setup: (context: Context) => Options<Id, S, G, A>): Store<Id, S, G, A>;
declare function defineOptionsStore<Id extends string, S extends Record<string | symbol, any>, G extends Getters<{}>, A extends Actions>({ id, state, getters, actions }: Options<Id, S, G, A>, ctx?: Store<Id, S, G, A>): Store<Id, S, G, A>;
declare function defineSetupStore<Id extends string, S extends Record<string | symbol, any>, G extends Getters<{}>, A extends Actions, Context extends Store<Id, S, G, A>>(setup: (context: Context) => Options<Id, S, G, A>): Store<Id, S, G, A>;

export { Accessor, Actions, Computes, DependenciesType, EffectFunction, Getter, Getters, Option, Options, Reactive, ReactiveMap, ReactiveType, ReadonlyReactive, Setter, Signal, Store, clear, createProxy, createReactive, createSignal, defineOptionsStore, defineSetupStore, defineStore, getId, parsePath, read, state, track, trigger, triggerChildren, triggerParent, useComputed, useEffect, useMemo, useReactive, watch, write };
